
/*
 * Copyright 2010 Google Inc.
 *
 * Use of this source code is governed by a BSD-style license that can be
 * found in the LICENSE file.
 */



#ifndef SkGrTexturePixelRef_DEFINED
#define SkGrTexturePixelRef_DEFINED

#include "SkGrPixelRef.h"

typedef SkGrPixelRef SkGrTexturePixelRef;
typedef SkGrPixelRef SkGrRenderTargetPixelRef;

#endif
